SystemOrganization addCategory: #CollectionExtensions!

!SequenceableCollection methodsFor: '*CollectionExtensions' stamp: 'TudorGirba 2/25/2012 18:18'!
pairsDistanceFrom: aSequenceableCollection
	
	self deprecated: 'Please use pairsSimilarityWith:'.
	^ self pairsSimilarityWith: aSequenceableCollection! !

!SequenceableCollection methodsFor: '*CollectionExtensions' stamp: 'TudorGirba 2/25/2012 18:18'!
pairsSimilarityWith: aSequenceableCollection
	
	"
		Examples:
		'1234' pairsSimilarityWith: '2234' ==> (2/3)
		'1234' pairsSimilarityWith: '123' ==>  (4/5)
		'1234' pairsSimilarityWith: '5678' ==> 0
	"
	| set1 set2 |
	set1 := Set new.
	set2 := Set new.
	self overlappingPairsDo: [:a :b | 
		set1 add: a -> b].
	aSequenceableCollection overlappingPairsDo: [:a :b | 
		set2 add: a -> b].
	^ 2 * (set1 intersection: set2) size / (set1 size + set2 size)! !

!SequenceableCollection methodsFor: '*CollectionExtensions' stamp: 'tg 4/26/2010 21:54'!
piecesCutWhere: testBlock do: enumerationBlock 
	"Evaluate testBlock for successive pairs of the receiver elements,
	breaking the receiver into pieces between elements where
	the block evaluated to true, and evaluate enumerationBlock
	with each of those pieces."
	"'A sentence. Another sentence... Yet another sentence.'
		piecesCutWhere: [:each :next | each = $. and: [next = Character space]] 
		do: [:each | Transcript show: each printString; cr]"

	| start prevElem nextElem size |
	self isEmpty ifTrue: [^self].
	prevElem := self at: (start := 1).
	2 to: (size := self size) do:
		[:index|
		 nextElem := self at: index.
		(testBlock value: prevElem value: nextElem) ifTrue: 
			[enumerationBlock value: (self copyFrom: start to: index - 1).
			 start := index].
		prevElem := nextElem].
	enumerationBlock value: (self copyFrom: start to: size)! !

!SequenceableCollection methodsFor: '*CollectionExtensions' stamp: 'TestRunner 10/25/2009 01:27'!
runsFailing: aBlock
	"Evaluate testBlock with the receiver elements, selecting from the
	receiver runs, that is sequences of adjacent elements, for which
	the block returned false. Return an OrderedCollection of those runs."
	"'Hello to\all of the world,\isn''t Smalltalk cool?' 
		runsFailing: [:each | each = $\] "

	^self runsSatisfying: [:each | (aBlock value: each) not]! !

!SequenceableCollection methodsFor: '*CollectionExtensions' stamp: 'TestRunner 10/25/2009 01:27'!
runsSatisfying: testBlock
	"Evaluate testBlock with the receiver elements, selecting from the
	receiver runs, that is sequences of adjacent elements, for which the block 
	returned true. Return an OrderedCollection of those subsequences."
	"'Hello to\all of the world,\isn''t Smalltalk cool?' 
		runsSatisfying: [:each | each ~= $\] "

	| runs |
	runs := OrderedCollection new.
	self runsSatisfying: testBlock do: [:each | runs add: each].
	^runs! !

!SequenceableCollection methodsFor: '*CollectionExtensions' stamp: 'TestRunner 10/25/2009 01:28'!
runsSatisfying: testBlock do: enumerationBlock 
	"Evaluate testBlock with the receiver elements, selecting from the
	receiver runs, that is sequences of adjacent elements, for which the block 
	returned true, and evaluate enumerationBlock with each of those 
	subsequences."
	"'Hello to\all of the world,\isn''t Smalltalk cool?' 
		runsSatisfying: [:each | each ~= $\] 
		do: [:each | Transcript show: each; cr]"

	| size start wasInside nowInside |
	start := 1.
	wasInside := false.
	1 to: (size := self size) do:
		[:index|
		 nowInside := testBlock value: (self at: index).
		 nowInside ~~ wasInside ifTrue:
			[wasInside := nowInside.
			 nowInside
				ifTrue: [start := index]
				ifFalse: [enumerationBlock value: (self copyFrom: start to: index - 1)]]].
	wasInside ifTrue: [enumerationBlock value: (self copyFrom: start to: size)]! !

!SequenceableCollection methodsFor: '*CollectionExtensions' stamp: 'TestRunner 10/25/2009 01:17'!
sliceFrom: startIndex

	"Answers a copy of a subset of the receiver, starting from element at start index 
	up to the end of the collection.

	Comfortable alternative to stupid copyFrom:to: method.
	See also comment on sliceFrom:to:"

	^self sliceFrom: startIndex to: self size! !

!SequenceableCollection methodsFor: '*CollectionExtensions' stamp: 'TestRunner 10/25/2009 01:16'!
sliceFrom: startIndex to: endIndex

	"Answers a copy of a subset of the receiver, starting from element at start index
	until element at end index.

	Comfortable alternative to stupid copyFrom:to: method.
	If the start or end index is negative, the index will be counted from the end of the collection.

	Note: this method does NEVER throw a SubscriptOutOfBoundsError. If the indexes are
	too large or start is greater than end, then an empty collection will be returned."

	| start end |

	start := 1 max:
		(startIndex positive ifTrue: [startIndex] ifFalse: [self size + startIndex +1]).

	end := self size min: 
		(endIndex positive ifTrue: [endIndex] ifFalse: [self size + endIndex]).

	((start > end) or: [end <= 0]) ifTrue: [^self copyEmpty: 0].
	^self copyFrom: start to: end! !

!SequenceableCollection methodsFor: '*CollectionExtensions' stamp: 'TestRunner 10/25/2009 01:17'!
sliceTo: endIndex
	"Answer a copy of a subset of the receiver, starting from first element 
	up to element at endIndex."

	^self sliceFrom: 1 to: endIndex! !

!OrderedCollection methodsFor: '*CollectionExtensions' stamp: 'simon.denier 3/26/2010 16:18'!
overlappingPairsCollect: aBlock 
	"Answer the result of evaluating aBlock with all of the overlapping pairs of my elements. Override superclass in order to use addLast:, not at:put:."
	| retval |
	retval := self species new: self size - 1.
	firstIndex to: lastIndex - 1 do:
		[:index |
		retval addLast: (aBlock value: (array at: index) value: (array at: index + 1))].
	^retval
! !

!OrderedCollection methodsFor: '*CollectionExtensions' stamp: 'TestRunner 10/25/2009 01:30'!
removeAtIndex: anIndex
	"Remove the element of the collection at position anIndex.  Answer the object removed."

	| obj |
	obj := self at: anIndex.
	self removeIndex: anIndex + firstIndex - 1.
	^obj! !

!String methodsFor: '*CollectionExtensions' stamp: 'on 8/24/2008 12:21'!
asEnglishPlural
	"Answer the plural of the receiver.  Assumes the receiver is an English noun.
	For a more comprehensive algorithm please refer to ''An Algorithmic Approach
	to English Pluralization'' by Damian Conway."

	self size < 2 ifTrue: [ ^self ].
	self asString = 'child' ifTrue: [ ^'children' ].

	((self last == $y) and: [ (self at: self size - 1) isVowel not ]) 
		ifTrue: [ ^self replaceSuffix: 'y' with: 'ies' ].
	(self endsWith: 's') ifTrue: [ ^self replaceSuffix: 's' with: 'ses' ].

	^self copyWith: $s.! !

!String methodsFor: '*CollectionExtensions' stamp: 'simondenier 2/4/2011 23:00'!
deepFlattenInto: stream

	stream nextPut: self! !

!String methodsFor: '*CollectionExtensions' stamp: 'TudorGirba 7/19/2011 19:10'!
intervalFromStartLine: aStartLine startColumn: aStartColumn toEndLine: anEndLine endColumn: anEndColumn

	" Returns an interval spanning between startLine @ startColumn to endLine @ endColumn"
	
	| lineIndex intervalStart intervalEnd |
	lineIndex := 0.
	intervalStart := 1.
	intervalEnd := 0.
	self lineIndicesDo: [ :start :endWithoutDelimiters :end |
		lineIndex := lineIndex + 1.
		lineIndex = aStartLine ifTrue: [ intervalStart := start + aStartColumn - 1 ].
		lineIndex = anEndLine ifTrue: [ intervalEnd := start + anEndColumn - 1] ].
	^ intervalStart to: intervalEnd! !

!String methodsFor: '*CollectionExtensions' stamp: 'TudorGirba 1/19/2012 13:34'!
intervalOfLine: aLineNumber

	"Answer an interval spanning between the first and the last character of the line from aLineNumber"
	
	| lineIndex |
	lineIndex := 0.
	self lineIndicesDo: [ :start :endWithoutDelimiters :end |
		lineIndex := lineIndex + 1.
		lineIndex = aLineNumber ifTrue: [ ^ start to: end ] ].
	^ 0 to: 0! !

!String methodsFor: '*CollectionExtensions' stamp: 'TudorGirba 1/19/2012 13:34'!
intervalOfLineCorrespondingToIndex: anIndex

	"Answer an interval spanning between the first and the last character of the line containing the given character index"
	
	self lineIndicesDo: [:start :endWithoutDelimiters :end |
		anIndex <= end ifTrue: [^ start to: end]].
	^ 0 to: 0! !

!String methodsFor: '*CollectionExtensions' stamp: 'tg 4/26/2010 21:53'!
piecesCutWhere: aBlock 
	"Evaluate testBlock for successive pairs of the receiver elements,
	breaking the receiver into pieces between elements where
	the block evaluated to true, and return an OrderedCollection of
	those pieces."
	"'A sentence. Another sentence... Yet another sentence.'
		piecesCutWhere: [:each :next | each = $. and: [next = Character space]]"

	| pieces |
	pieces := OrderedCollection new.
	self piecesCutWhere: aBlock do: [:each | pieces add: each].
	^pieces! !

!String methodsFor: '*CollectionExtensions' stamp: 'tg 6/27/2010 13:08'!
piecesCutWhereCamelCase
	"Breaks apart words written in camel case.
	
	It's not simply using piecesCutWhere: because we want
	to also deal with abbreviations and thus we need to 
	decide based on three characters, not just on two:
	 ('FOOBar') piecesCutWhereCamelCase asArray = #('FOO' 'Bar').
	('FOOBar12AndSomething') piecesCutWhereCamelCase asArray = #('FOO' 'Bar' '12' 'And' 'Something')	
	"
	
	| start previous current next pieces |
	self isEmpty ifTrue: [^self].
	start := 1.
	pieces := OrderedCollection new.
	3 to: self size do: [ :index |
		previous := self at: index - 2.
		current := self at: index - 1.
		next := self at: index.
		((previous isLowercase and: [current isUppercase]) or: [
			(previous isUppercase and: [current isUppercase and: [next isLowercase ]]) or: [
				(previous isDigit not and: [current isDigit]) or: [
					previous isDigit and: [current isDigit not] 
		]]]) ifTrue: [
			pieces add: (self copyFrom: start to: index - 2).
			start := index - 1].
	].
	pieces addAll: ((self copyFrom: start to: self size) piecesCutWhere: [:a :b | 
		(a isDigit and: [b isDigit not]) or: [a isDigit not and: [b isDigit ]]]).
	^pieces! !

!String methodsFor: '*CollectionExtensions' stamp: 'simon.denier 6/9/2009 15:53'!
removePrefix: prefix
	"Remove the given prefix, if present."

	^(self beginsWith: prefix) ifTrue: [ self copyFrom: 1 + prefix size to: self size ] ifFalse: [ self ]! !

!String methodsFor: '*CollectionExtensions' stamp: 'tg 6/25/2008 13:04'!
removeSuffix: suffix
	"Remove the given suffix, if present."

	^(self endsWith: suffix) ifTrue: [ self copyFrom: 1 to: self size - suffix size ] ifFalse: [ self ]! !

!String methodsFor: '*CollectionExtensions' stamp: 'on 8/24/2008 12:22'!
replaceSuffix: suffix with: replacement
	^ (self removeSuffix: suffix), replacement! !

!Symbol methodsFor: '*CollectionExtensions' stamp: 'stephane.ducasse 10/14/2008 22:07'!
value
	"Allow this object to act as a ValueHolder on itself."

	^self! !

TestCase subclass: #CECollectionExtensionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionExtensions'!

!CECollectionExtensionTest methodsFor: 'collect as set' stamp: 'stephane.ducasse 10/14/2008 22:15'!
testCollectAsSet
	"self debug: #testCollectAsSet"
	
	self assert: ((#() collectAsSet: [:each | each odd]) = Set new).
	self assert: (#(1 2 3 4 5 6) collectAsSet: [:each | each odd])
					 = (Set with: true with: false).
	self assert: (#(1 3 5 7 9 11) collectAsSet: [:each | each odd]) 
					= (Set with: true).
					
	self assert: (#(1 2 3 4 5 4 3 2 1) collectAsSet: [:each | each]) = (1 to: 5) asSet.
	
	! !

!CECollectionExtensionTest methodsFor: 'collect as set' stamp: 'stephane.ducasse 10/14/2008 22:08'!
testCollectAsSetUsingSymbol
	"self debug: #testCollectAsSetUsingSymbol"
	
	self assert: ((#() collectAsSet: #odd) = Set new).
	self assert: (#(1 2 3 4 5 6) collectAsSet: #odd)
					 = (Set with: true with: false).
	self assert: (#(1 3 5 7 9 11) collectAsSet: #odd) 
					= (Set with: true).! !

!CECollectionExtensionTest methodsFor: 'flatten' stamp: 'simondenier 2/4/2011 22:53'!
testDeepFlatten

	self assert: #(1 2 3)
		equals: #((1) (2) (3)) deepFlatten.
	self assert: #(1 2 3 1 2)
		equals: #((1 2) (3 1 2)) deepFlatten.
! !

!CECollectionExtensionTest methodsFor: 'flatten' stamp: 'simondenier 2/4/2011 23:01'!
testDeepFlattenIsRecursive

	self assert: #(1 2 4 5 3)
		equals: #((1 2) ((4 5) 3)) deepFlatten.! !

!CECollectionExtensionTest methodsFor: 'flatten' stamp: 'simondenier 2/4/2011 23:04'!
testDeepFlattenOnFlatCollection

	self assert: #(1 2 4)
		equals: #(1 2 4) deepFlatten.
	self assert: #(5 3)
		equals: #(5 ((3))) deepFlatten.! !

!CECollectionExtensionTest methodsFor: 'flatten' stamp: 'simondenier 2/4/2011 23:03'!
testDeepFlattenOnString
	"don't flatten strings"

	self assert: #(a b c d e)
		equals: #((a b) ((c d) e)) deepFlatten.
	self assert: #('foo' 'bar' 'zorg')
		equals: #(('foo' ('bar')) 'zorg') deepFlatten! !

!CECollectionExtensionTest methodsFor: 'as yet unclassified' stamp: 'tg 7/10/2010 16:29'!
testDetectIfOne
	| element result |
	result := #(1 2 3) detect: [:each | each = 2] ifOne: [:theOne | element := theOne ].
	self assert: element = 2.
	self assert: result = 2.

	element := nil.
	result := #(1 2 3) detect: [:each | each = 4] ifOne: [:theOne | element := theOne ].
	self assert: element isNil.
	self assert: result isNil.! !

!CECollectionExtensionTest methodsFor: 'as yet unclassified' stamp: 'tg 7/10/2010 16:27'!
testDetectIfOneIfNone
	| element |
	#(1 2 3) detect: [:each | each = 2] ifOne: [:theOne | element := theOne ] ifNone: [element := nil].
	self assert: element = 2.

	#(1 2 3) detect: [:each | each = 4] ifOne: [:theOne | element := theOne ] ifNone: [element := nil].
	self assert: element isNil.
! !

!CECollectionExtensionTest methodsFor: 'flat collect' stamp: 'stephane.ducasse 10/14/2008 23:06'!
testFlatCollect
	"self debug: #testFlatCollect"
	
	! !

!CECollectionExtensionTest methodsFor: 'flat collect' stamp: 'simon.denier 6/4/2010 17:50'!
testFlatCollectArray
	"self debug: #testFlatCollectArray"
	
	self assert: ((#((1 2) (3 4) (5 3)) flatCollect: [ :each | each ]) = #(1 2 3 4 5 3)).
	self assert: ((#((1 2) (2 3) (1 3 4)) flatCollect: [:each | each]) = #(1 2 2 3 1 3 4)). 	
		
	self assert: ((#((1 2) (2 3) () ()) flatCollect: [:each | each]) = #(1 2 2 3)). 

	self assert: ((#((1 2) (2 3) (1 3 4)) flatCollect: [:each| Array with: each])
					=  #(#(1 2) #(2 3) #(1 3 4))).
					
	self assert: ((#((1 2) (2 3) (1 3 4)) flatCollect: [:each| Set with: each])
					=  #(#(1 2) #(2 3) #(1 3 4))).	
	
! !

!CECollectionExtensionTest methodsFor: 'flat collect' stamp: 'simon.denier 6/4/2010 17:51'!
testFlatCollectSet
	"self debug: #testFlatCollectSet"
	
	self assert: ((#((1 2) (1 2) (1 3 4)) asSet  flatCollect: [:each | each]) = #(1 1 2 3 4) asSet).
	self assert: ((#() asSet flatCollect: [:each]) = #() asSet).

	self assert:  ((#((1 2) () (1 3 4)) asSet  flatCollect: [:each | each]) = #(1 1 2 3 4) asSet).
	self assert:  ((#((1 2) #((99)) (1 3 4)) asSet  flatCollect: [:each | each]) 
					= #(1 1 2 3 4 (99)) asSet).
	self assert:  ((#((1 2) #(()) (1 3 4)) asSet  flatCollect: [:each | each]) 
					= #(1 1 2 3 4 ()) asSet).! !

!CECollectionExtensionTest methodsFor: 'flatten' stamp: 'simondenier 2/4/2011 22:51'!
testFlatten

	self assert: #(1 2 3)
		equals: #((1) (2) (3)) flatten.
	self assert: #(1 2 3 1 2)
		equals: #((1 2) (3 1 2)) flatten.
	self assert: #(a b (c d))
		equals: #((a b) ((c d))) flatten.
	self should: [ #((1 2) 3) flatten ] raise: Error.! !

!CECollectionExtensionTest methodsFor: 'groupedBy' stamp: 'stephane.ducasse 10/14/2008 22:25'!
testGroupedBy
	"self debug: #testGroupedBy"
	
	| res |
	res := #(1 2 3 4 5) asOrderedCollection 
				groupedBy: [:each | each odd].
	self assert:   (res at: true) = #(1 3 5) asOrderedCollection.
	self assert: (res at: false) = #(2 4) asOrderedCollection! !

!CECollectionExtensionTest methodsFor: 'groupedBy' stamp: 'stephane.ducasse 10/14/2008 22:23'!
testGroupedByArray
	"self debug: #testGroupedByArray"
	
	| res |
	res := #(1 2 3 4 5) groupedBy: [:each | each odd].
	self assert:   (res at: true) = #(1 3 5).
	self assert: (res at: false) = #(2 4)! !

!CECollectionExtensionTest methodsFor: 'groupedBy' stamp: 'stephane.ducasse 10/14/2008 22:24'!
testGroupedBySet
	"self debug: #testGroupedBySet"
	
	| res |
	res := #(1 2 3 4 5 3 4 5) asSet groupedBy: [:each | each odd].
	self assert: (res at: true) = #(1 3 5) asSet.
	self assert: (res at: false) = #(2 4) asSet! !

!CECollectionExtensionTest methodsFor: 'symbol - value' stamp: 'stephane.ducasse 10/14/2008 22:10'!
testSymbolInPlaceOfBlock
	"self debug: #testSymbolInPlaceOfBlock"
	
	self assert: (#(1 2 3 4) collect: #odd) =  #(true false true false).
	self assert: (#(1 2 3 4) select: #odd) =  #(1 3).! !

TestCase subclass: #CESequenceableCollectionExtensionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionExtensions'!

!CESequenceableCollectionExtensionTest methodsFor: 'tests' stamp: 'TudorGirba 2/25/2012 18:19'!
testPairsSimilarityWith
	
	self assert: ('1234' pairsSimilarityWith: '2234') equals: (2/3).
	self assert: ('1234' pairsSimilarityWith: '123') equals: (4/5).
	self assert: ('1234' pairsSimilarityWith: '5678') equals: 0! !

TestCase subclass: #CEStringExtensionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionExtensions'!

!CEStringExtensionTest methodsFor: 'tests' stamp: 'TudorGirba 7/19/2011 19:59'!
testIntervalFromStartLineStartColumnToEndLineEndColumn
	| string |
	string := '123
56
89'.
	self assert: (string intervalFromStartLine: 1 startColumn: 1 toEndLine: 1 endColumn: 2) = (1 to: 2).
	self assert: (string intervalFromStartLine: 2 startColumn: 2 toEndLine: 3 endColumn: 2) = (6 to: 9)! !

!CEStringExtensionTest methodsFor: 'tests' stamp: 'TudorGirba 11/23/2011 16:14'!
testIntervalOfLine
	| string |
	string := '123
56
89'.
	self assert: (string intervalOfLine: 1) = (1 to: 4).
	self assert: (string intervalOfLine: 2) = (5 to: 7).
	self assert: (string intervalOfLine: 3) = (8 to: 9)! !

!CEStringExtensionTest methodsFor: 'tests' stamp: 'TudorGirba 1/19/2012 14:08'!
testIntervalOfLineCorrespondingToIndex
	| string |
	string := '123
56
89'.
	self assert: (string intervalOfLineCorrespondingToIndex: 1) = (1 to: 4).
	self assert: (string intervalOfLineCorrespondingToIndex: 7) = (5 to: 7).
	self assert: (string intervalOfLineCorrespondingToIndex: 9) = (8 to: 9)! !

!CEStringExtensionTest methodsFor: 'tests' stamp: 'tg 6/27/2010 13:06'!
testPiecesCutWhereCamelCase

	self assert: ('' piecesCutWhereCamelCase) isEmpty.
	self assert: ('fBar' piecesCutWhereCamelCase) asArray  = #('f' 'Bar').
	self assert: ('fooBar' piecesCutWhereCamelCase) asArray  = #('foo' 'Bar').
	self assert: ('FOOBar') piecesCutWhereCamelCase asArray = #('FOO' 'Bar').
	self assert: ('fooBar1' piecesCutWhereCamelCase) asArray  = #('foo' 'Bar' '1').
	self assert: ('FOOBar12AndSomething') piecesCutWhereCamelCase asArray = #('FOO' 'Bar' '12' 'And' 'Something').! !

!Set methodsFor: '*CollectionExtensions' stamp: 'jannik.laval 2/6/2009 16:44'!
flatCollect: aBlock 
	
	
	^self flatCollectAsSet: aBlock! !

!Collection methodsFor: '*CollectionExtensions' stamp: 'jannik.laval 1/28/2009 17:30'!
collectAsSet: aBlock
	"Evaluates aBlock for each element of the receiver and collects
	the resulting values into a Set."

	"This is an efficient shorthand for [ (self collect: aBlock) asSet ]."
	"originally developed by a. kuhn and released under MIT."
	
	^self inject: Set new into: [ :set :each | 
		set add: (aBlock value: each); yourself ].! !

!Collection methodsFor: '*CollectionExtensions' stamp: 'tg 10/25/2009 01:19'!
copyEmpty: aSize
	"Answer a copy of the receiver that contains no elements.

	This method should be redefined in subclasses that add
	instance variables, so that the state of those variables
	is preserved"

	^self class new: aSize! !

!Collection methodsFor: '*CollectionExtensions' stamp: 'simondenier 2/4/2011 22:55'!
deepFlatten

	^ Array streamContents: [ :stream |
			self do: [ :each | each deepFlattenInto: stream ]]! !

!Collection methodsFor: '*CollectionExtensions' stamp: 'simondenier 2/4/2011 22:57'!
deepFlattenInto: stream

	self do: [ :each | each deepFlattenInto: stream ]! !

!Collection methodsFor: '*CollectionExtensions' stamp: 'tg 7/10/2010 11:02'!
detect: aBlock ifOne: presentBlock
	^ self detect: aBlock ifOne: presentBlock ifNone: [nil] ! !

!Collection methodsFor: '*CollectionExtensions' stamp: 'tg 7/10/2010 11:01'!
detect: aBlock ifOne: presentBlock ifNone: noneBlock
	| result |
	result := self detect: aBlock ifNone: [nil].
	^ result isNil 
		ifTrue: [ noneBlock value ]
		ifFalse: [ presentBlock value: result ]! !

!Collection methodsFor: '*CollectionExtensions' stamp: 'jannik.laval 1/28/2009 08:04'!
equalsTo: aCollection 
	"Answer true if the reciever contains the same elements as aCollection, and vice versa."

	^(aCollection isCollection) 
		and: [ aCollection size = self size 
			and: [ aCollection allSatisfy: [ :each | 
				(aCollection occurrencesOf: each) = (self occurrencesOf: each) ]]]! !

!Collection methodsFor: '*CollectionExtensions' stamp: 'simon.denier 4/1/2009 15:19'!
flatCollect: aBlock 
	"Evaluate aBlock for each of the receiver's elements and answer the
	list of all resulting values flatten one level. Assumes that aBlock returns some kind
	of collection for each element. Equivalent to the lisp's mapcan"
	"original written by a. Kuhn and released under MIT"
	
	| stream |
	self isEmpty ifTrue: [ ^ self copy ].
	stream := (self species new: 0) nsWriteStream.
	self do: [ :each | stream nextPutAll: (aBlock value: each) ].
	^ stream contents! !

!Collection methodsFor: '*CollectionExtensions' stamp: 'jannik.laval 8/27/2010 15:54'!
flatCollect: aBlock as: aCollectionClass
	"Evaluate aBlock for each of the receiver's elements and answer the
	list of all resulting values flatten one level. Assumes that aBlock returns some kind
	of collection for each element. Equivalent to the lisp's mapcan"
	
	| col |
	self isEmpty ifTrue: [^self copy ].
	col := aCollectionClass new: self size. 	
	self do: [ :each |
		col addAll: (aBlock value: each) ].
	^col! !

!Collection methodsFor: '*CollectionExtensions' stamp: 'jannik.laval 2/12/2009 14:06'!
flatCollectAsSet: aBlock
	"Evaluate aBlock for each of the receiver's elements and answer the
	list of all resulting values flatten one level. Assumes that aBlock returns some kind
	of collection for each element. Equivalent to the lisp's mapcan"
	
	"original written by a. Kuhn and released under MIT"
	
	| set |
	self isEmpty ifTrue: [^self copy ].
	set := Set new. 	
	self do: [ :each |
		set addAll: (aBlock value: each) ].
	^set! !

!Collection methodsFor: '*CollectionExtensions' stamp: 'tg 7/11/2010 14:27'!
flatten
	"Recursively collect each non-collection element of the receiver and its descendant
	collections.  Please note, this implementation assumes that strings are to be treated
	as objects rather than as collection."

	^self gather: [ :each | each ]! !

!Collection methodsFor: '*CollectionExtensions' stamp: 'stephane.ducasse 10/14/2008 22:28'!
groupedBy: aBlock
	"Return a dictionary whose keys are the result of evaluating aBlock for all elements in
	 the collection, and the value for each key is the collection of elements that evaluated
	 to that key. e.g.
	     #(1 2 3 4 5) groupedBy: [:each | each odd]
	   a Dictionary  
	     true ---> #( 1 3 5)
	     false --> #(2 4)
	originally developed by a. kuhn and released under MIT."

	| result |
	result := Dictionary new.
	self do:
		[:each | | key collection |
		key := aBlock value: each.
		collection := result at: key ifAbsentPut: [OrderedCollection new].
		collection add: each].
	self species ~~ OrderedCollection ifTrue:
		["Convert the result collections to be the right type.
		  Note that it should be safe to modify the dictionary
		  while iterating because we only replace values for existing keys"
		result keysAndValuesDo:
			[:key :value | result at: key put: (self species withAll: value)]].

	^result! !

!Collection methodsFor: '*CollectionExtensions' stamp: 'tg 4/26/2010 21:43'!
groupedBy: aBlock affect: anotherBlock
	"First, evaluate aBlock for each of the receiver's elements and group the
	elements by the resulting values, and second, evaluate anotherBlock for
	each of the resulting groups and return a dictionary with the first pass'
	results as keys and the second pass' results as values."

	"This is a shorthand for [ (self groupedBy: aBlock) collect: anotherBlock ]."

	^(self groupedBy: aBlock) associationsDo: [ :each | each value: (anotherBlock value: each value) ]! !

!Collection methodsFor: '*CollectionExtensions' stamp: 'jannik.laval 1/28/2009 16:37'!
includesAll: aCollection 
	"Answer true if the receiver includes all elements of aCollection with at
	least as many occurrences as in aCollection. For a less strict comparison
	please refer to supersetOf: and its inverse subsetOf:."


	^(aCollection isCollection) and: [
		aCollection size <= self size and: [
			aCollection allSatisfy: [ :each | 
				(aCollection occurrencesOf: each) <= (self occurrencesOf: each) ]]]! !

!Collection methodsFor: '*CollectionExtensions' stamp: 'tg 10/14/2009 08:47'!
nilSafeGroupedBy: aBlock
	^ self groupedBy: [ :each |
		| value |
		value := aBlock value: each.
		value ifNil: [ UndefinedObject ].
	]	! !

!Collection methodsFor: '*CollectionExtensions' stamp: 'Jannik.Laval 4/18/2009 17:47'!
selectAsSet: aBlock 
	"Evaluate aBlock with each of the receiver's elements as the argument. 
	Collect into a new set, only those elements for which
	aBlock evaluates to true.  Answer the new collection."

	| newSet |
	newSet := Set new.
	self do: [:each | (aBlock value: each) ifTrue: [newSet add: each]].
	^newSet! !

!Collection methodsFor: '*CollectionExtensions' stamp: 'tg 10/15/2009 11:59'!
shuffle
	"Swaps the receiver's elements at random."

	self shuffle: (self size * self size log) asInteger! !

!Collection methodsFor: '*CollectionExtensions' stamp: 'tg 10/15/2009 11:58'!
shuffle: times
	"Swaps random elements of the receiver."

	| size random |
	size := self size.
	random := Random new.
	times timesRepeat: [ 
		self swap: (random next * size) floor + 1 with: (random next * size) floor + 1 
	].! !

!Collection methodsFor: '*CollectionExtensions' stamp: 'simon.denier 5/20/2009 16:27'!
sum: aSymbolOrBlock 
	 
	^self 
		inject: 0 
		into: [:sum :each | sum + (aSymbolOrBlock value: each)]! !

!Object methodsFor: '*collectionextensions' stamp: 'simondenier 2/4/2011 22:57'!
deepFlattenInto: stream

	stream nextPut: self! !
